@@ -18,13 +18,15 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: build.h 3479 2009-01-17 17:59:20Z eht16 $
+ * $Id: build.h 3732 2009-04-24 19:31:00Z eht16 $
  */
 
 
 #ifndef GEANY_BUILD_H
 #define GEANY_BUILD_H 1
 
+#define GEANY_BUILD_ERR_HIGHLIGHT_MAX 100
+
 typedef enum	/* Geany Build Options */
 {
 	GBO_COMPILE,
@@ -41,6 +43,7 @@ typedef struct GeanyBuildInfo
 	gchar			*dir;
 	guint			 file_type_id;
 	gchar			*custom_target;
+	gint			 message_count;
 } GeanyBuildInfo;
 
 extern GeanyBuildInfo build_info;
