@@ -1,8 +1,8 @@
 /*
  *      geanywraplabel.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: geanywraplabel.c 2789 2008-07-18 13:45:23Z eht16 $
+ *  $Id: geanywraplabel.c 3708 2009-04-15 22:47:33Z eht16 $
  */
 
 /*
@@ -27,14 +27,11 @@
  */
 
 
-#include <gtk/gtklabel.h>
+#include <gtk/gtk.h>
+#include "utils.h"
 #include "geanywraplabel.h"
 
 
-/* Local data */
-static GObjectClass *parent_class = NULL;
-
-
 
 #define GEANY_WRAP_LABEL_GET_PRIVATE(obj)		(G_TYPE_INSTANCE_GET_PRIVATE((obj),\
 	GEANY_WRAP_LABEL_TYPE, GeanyWrapLabelPrivate))
@@ -56,44 +53,17 @@ typedef struct
 } GeanyWrapLabelPrivate;
 
 
-static void geany_wrap_label_class_init		(GeanyWrapLabelClass *klass);
-static void geany_wrap_label_init			(GeanyWrapLabel *self);
 static void geany_wrap_label_size_request	(GtkWidget *widget, GtkRequisition *req);
 static void geany_wrap_label_size_allocate	(GtkWidget *widget, GtkAllocation *alloc);
 static void geany_wrap_label_set_wrap_width	(GtkWidget *widget, gsize width);
 
-
-
-GType geany_wrap_label_get_type()
-{
-	static GType type = G_TYPE_INVALID;
-
-	if (G_UNLIKELY(type == G_TYPE_INVALID))
-	{
-		GTypeInfo gwl_info = {
-			sizeof (GeanyWrapLabelClass),
-			NULL, NULL,
-			(GClassInitFunc) geany_wrap_label_class_init,
-			NULL,
-			NULL,
-			sizeof (GeanyWrapLabel),
-			3,
-			(GInstanceInitFunc) geany_wrap_label_init,
-			NULL
-		};
-
-		type = g_type_register_static(GTK_TYPE_LABEL, "GeanyWrapLabel", &gwl_info, 0);
-	}
-
-	return type;
-}
+G_DEFINE_TYPE(GeanyWrapLabel, geany_wrap_label, GTK_TYPE_LABEL);
 
 
 static void geany_wrap_label_class_init(GeanyWrapLabelClass *klass)
 {
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS(klass);
 
-	parent_class = g_type_class_peek_parent(klass);
 	widget_class->size_request = geany_wrap_label_size_request;
 	widget_class->size_allocate = geany_wrap_label_size_allocate;
 
@@ -148,7 +118,7 @@ static void geany_wrap_label_size_request(GtkWidget *widget, GtkRequisition *req
 /* Sets the wrap width to the width allocated to us. */
 static void geany_wrap_label_size_allocate(GtkWidget *widget, GtkAllocation *alloc)
 {
-	(* GTK_WIDGET_CLASS(parent_class)->size_allocate)(widget, alloc);
+	(* GTK_WIDGET_CLASS(geany_wrap_label_parent_class)->size_allocate)(widget, alloc);
 
 	geany_wrap_label_set_wrap_width(widget, alloc->width);
 }
@@ -167,7 +137,7 @@ GtkWidget *geany_wrap_label_new(const gchar *text)
 {
 	GtkWidget *l = g_object_new(GEANY_WRAP_LABEL_TYPE, NULL);
 
-	if (text != NULL && text[0] != '\0')
+	if (NZV(text))
 		gtk_label_set_text(GTK_LABEL(l), text);
 
 	pango_layout_set_wrap(gtk_label_get_layout(GTK_LABEL(l)), PANGO_WRAP_WORD_CHAR);
